import sys
import time

from .users import Mail
from .utils import Xml
from .utils import load_proxy
from FlowWork.Net.flownet import  FLowNet
from termcolor import cprint


PAYLOAD = """
https://twitter.com/signup
INPUT#full-name/I'{name}'
INPUT#email/I'{email}'
INPUT#password/I'{passwd}'
.sign-up-box/C
2
.skip-link/C
1
li>button/C
.sign-up-box/C
.StartCongratulations-buttons>a/C
1
.PillList-item>input/C
.StartActions>button/C
.js-skip-step/C
1
.StartActions>button/C
.js-close[Not now]/C
1
.js-close[Skip all]/C
.resend-confirmation-email-link/C
[over]"""


def register_one(start_time=None,f=None, m=None):
    if not start_time:
        start_time = time.time()
    if not m:
        m = Mail()

    cprint("Email:" + m.address , 'green')
    name = m.address.split("@")[0]
    passwd = m.address[:-1]
    email = m.address
    cprint("Generate account:"+ " | ".join([name, email, passwd]), "red")
    cprint("patch scripts ...", 'red')
    text= PAYLOAD
    text = text.format(name=name, email=email, passwd = passwd).split("\n")
    cprint("Initialize Browser .... ",'red', end='')
    if not f:
        # proxy_one = load_proxy(type_p='socks5')[0]
        proxy_one = "socks5://127.0.0.1:1080"
        f = FLowNet(proxy=proxy_one, driver='chrome')
    time.sleep(1)

    cprint(" Ok", 'green', attrs=['bold'])
    cprint("........................................", 'blue')
    try:
        for cmd in text:
            if not cmd.strip():continue
            f.flow_doc(cmd)
    except Exception as e:
        cprint(str(e), "red")
        f.phantom.close()
    else:
        with open("account.txt", "a") as fp:
            fp.write(":".join([name, email, passwd])+ "\n")
        cprint("Confirm .... ", 'yellow', end='')
        while 1:
            m.table()
            res = m.read('Confirm')
            if not res:
                cprint(".", 'yellow', end='')
                continue
            tmp =  Xml(res.text).text
            hs = tmp.find("replace(\"")
            if hs != -1:
                e_str = tmp[hs + 9: -2].replace("\\", "")
            f.go(e_str)
            break
    f.phantom.close()
    return email, passwd

def main():
    start_time = time.time()
    m = Mail()
    f = FLowNet(proxy='socks5://127.0.0.1:1080', driver='chrome')
    email, passwd = register_one()
    cprint("[registe  Over ... confirm account ..]", 'red')

    while 1:
        c = input("[press to continue]").strip()
        if c == 'q':
            cprint("Bye ~~")
            sys.exit(0)
        now = (time.time() - start_time) / 60
        if now > 9:
            m.more_10_minute()
        m.table()
        res = m.read(0)
        if not res:
            cprint("........................................", 'blue')
            continue
        tmp =  Xml(res.text).text
        hs = tmp.find("replace(\"")
        if hs != -1:
            e_str = tmp[hs + 9: -2].replace("\\", "")
            cprint("Go ? : " + e_str, 'green')
            c = input(" >>>>>[Y/no] Y")
            if not c or c == 'Y':
                f.go(e_str)

        cprint("........................................", 'yellow')

if __name__ == '__main__':
    main()


import asyncio
import socket
import time
import traceback


async def tcp_echo_client(num, host, loop):
    h, p = host.split(":")
    try:
        st = time.time()
        reader, writer = await asyncio.open_connection(h, int(p), loop=loop)
        et = time.time() -st
        # print('Close the socket')
        writer.close()
        return host,et

    except socket.error as e:
        # traceback.print_stack()
        return host,9999
    # print('Send: %r' % message)
    # writer.write(message.encode())

    # data = yield from reader.read(100)
    # print('Received: %r' % data.decode())


async def _Tests(hosts, loop):
    
    task = [tcp_echo_client(i, host, loop) for i, host in enumerate(hosts)]
    return await asyncio.gather(*task)



def Tests(hosts):
    loop = asyncio.get_event_loop()
    res = loop.run_until_complete(_Tests(hosts, loop))
    sorted(res, key=lambda x:x[1])
    loop.close()
    return res


async def handle_echo(reader, writer):
    data = await reader.read(100)
    message = data.decode()
    addr = writer.get_extra_info('peername')
    print("Received %r from %r" % (message, addr))

    print("Send: %r" % message)
    writer.write(data)
    await writer.drain()

    print("Close the client socket")
    writer.close()


def main():
    loop = asyncio.get_event_loop()
    coro = asyncio.start_server(handle_echo, '127.0.0.1', 18888, loop=loop)
    server = loop.run_until_complete(coro)

    # Serve requests until Ctrl+C is pressed
    print('Serving on {}'.format(server.sockets[0].getsockname()))
    try:
        loop.run_forever()
    except KeyboardInterrupt:
        pass

    # Close the server
    server.close()
    loop.run_until_complete(server.wait_closed())
    loop.close()


import asyncio
import os, sys
import time
import socket
import configparser
import json
import urllib.parse as up

import async_timeout
import aiosocks
import aiohttp

from termcolor import cprint
from requests import Session
from bs4 import BeautifulSoup
from aiosocks.connector import ProxyConnector, ProxyClientRequest


def load_config():
    c = configparser.ConfigParser()
    c.read(os.path.join("/usr/local/etc/SocialOrigan", "config.ini"))
    return c


Config = load_config()
ROOT = Config.get("path","ROOT")

def load_proxy(default=False, type_p=None):
    c = configparser.ConfigParser()
    if not os.path.exists(os.path.join(ROOT, "Proxy.ini")) or default:
        c.read(os.path.join(ROOT, "config.ini"))
        ProxyHostPort = c.get("proxy config", 'hostport')
        ProxyType = c.get("proxy config", 'type')
        return ProxyType + "://" + ProxyHostPort

    c.read(os.path.join(ROOT, "Proxy.ini"))
    proxies = []
    for i in range(len(c.sections())):
        s = c.get(str(i), 'hostport')
        t = c.get(str(i), 'type')
        if type_p:
            if type_p != t.lower().strip().split()[0]:
                continue

        if ' ' in t:
            t = t.strip().split()[0]

        proxies.append(t.lower()+"://"+s)

    return proxies

## session area
# sess will be a global variable
#  :: change , create, switch
#  
headers = {}
headers['User-Agent'] = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.186 Safari/537.36'

default_proxy = load_proxy(default=True)

Proxies = {
    'http': default_proxy,
    'https': default_proxy,
}


def Xml(resp):
    if hasattr(resp, 'content'):
        return BeautifulSoup(resp.text, 'lxml')
    else:
        return BeautifulSoup(resp, 'lxml')


######################
# check proxy if connected!
try:
    s = socket.socket()
    s.settimeout(7)
    _,host, port = default_proxy.split(":")
    host = host[2:]
    s.connect((host, int(port)))
except Exception as e:
    print(e)
    print("Local Proxy not Connected!", host, port)
    sys.exit(0)



class Sess:
    """
    auto_change_proxy_time: int
        timer can auto change proxy
    proxy: True/ str /list,
    label: a key to swich sess
    """
    _data = {}
    register_auto_change_proxy_key = set()

    def __init__(self, label, proxy=True, auto_change_proxy_time=0):
        self.sess = Session()
        self.label = label
        self.now_time = time.time()
        self.auto_change_proxy_time = auto_change_proxy_time
        
        self.sess.headers.update(headers)

        if proxy is True:
            self.sess.proxies.update(Proxies)
        elif isinstance(proxy, str):
            pro = {}
            t,h = proxy.split("://")
            if t == "socks5":
                proxy = t +"h://" + h
            pro['http'] = proxy
            pro['https'] = proxy
            self.sess.proxies.update(pro)

        elif isinstance(proxy, (list, tuple)):
            proxy['http'] = proxy[0]+ "://" + proxy[1]
            proxy['https'] = proxy[0]+ "://" + proxy[1]
            self.sess.proxies.update(proxy)

        self.Domain = ''

        if self.auto_change_proxy_time > 0:
            
            self.__class__.register_auto_change_proxy_key.add(self.label)


        self.__class__._data[label] = self

    def __del__(self):
        if self.label in self.__class__.register_auto_change_proxy_key:
            self.__class__.register_auto_change_proxy_key.remove(self.label)
        if self.label in self.__class__._data:
            del self.__class__._data[self.label]


    def set_proxy(self, proxy):
        n = time.time()
        if self.auto_change_proxy_time == 0:
            return False
        
        
        if (n - self.now_time) > self.auto_change_proxy_time:
            proxy = proxy.replace("5://","5h://")
            cprint("[+] proxy to " + self.label + "|" + proxy, 'green')
            self.sess.proxies['http'] = proxy
            self.sess.proxies['https'] = proxy
            self.now_time = n
        else:
            return False

    def test(self):
        ip = self.Get("http://ipv4.icanhazip.com").text
        # w = self.Get('http://int.dpool.sina.com.cn/iplookup/iplookup.php?format=js&ip=' + ip).json()
        # w = json.loads(w.split("=")[1][:-1])
        cprint('ip :' + ip, 'red')
        # print(w)
        # cprint('cou :' + w['country'], 'green')
        # cprint('province :' + w['province'], 'green')

    @classmethod
    def auto_change_proxy(cls):
        proxies = load_proxy()
        c = 0
        i = 0
        
        while c < len(cls.register_auto_change_proxy_key):
            p = proxies[i]
            sess = cls._data[list(cls.register_auto_change_proxy_key)[c]]
            c += 1
            if not sess.set_proxy(p):
                continue
            i+=1
                    

    def async_gets(self, urls, proxy=None):
        if not proxy:
            proxy = self.sess.proxies['https']
            proxy = proxy.replace('socks5h','socks5')

        res = Gets([[i, proxy] for i in urls])
        return res

    def async_downloads(self, urls, proxy=None):
        if not proxy:
            proxy = self.sess.proxies['https']
            proxy = proxy.replace('socks5h','socks5')

        res = Downloads([[[i, proxy], ''] for i in urls])
        return res


    def Get(self, url, **kargs):
        if self.auto_change_proxy_time > 0:
            self.__class__.auto_change_proxy()
        if not url.startswith("http"):
            url = up.urljoin(self.Domain, url)
        else:
            
            if  not self.Domain:
                p = up.urlparse(url)
                self.Domain = p.scheme + "://" + p.netloc

        return self.sess.get(url, **kargs)
        # return self.sess.get(url, verify=False,**kargs)

    def setDomain(self, domain):
        self.Domain = domain

    def Table(self, url, data=None, json=False):
        """
        json: xml  or json
        if data != None: will use Post, default use Get

        """
        if not hasattr(url, 'text') and isinstance(url, str):
            if data:

                res = self.Post(url, data)
            else:
                res = self.Get(url)

            if type == 'json':
                tmp = res.json()
                return tmp
            else:
                tmp = Xml(res.text)
        else:
            tmp = url

        
        tables = tmp.select("table")
        res = []
        for table in tables:
            rows = table.select("tr")
            headers = [col.text for col in rows[0].select("td") if hasattr(col, 'text')]
            if len(rows) <2:
                continue
            headers2 = [col.text for col in rows[1].select("td") if hasattr(col, 'text')]

            msgs = []
            for row in rows[1:]:
                values = [col.text for col in row.select("td") if hasattr(col, 'text') and col.text]
                if not values:
                    continue
                if len(values) != headers:
                    headers = headers2

                if len(values) / len(headers) < 0.3:
                    continue

                if values == headers:
                    continue
                msgs.append(dict(zip(headers, values)))

            yield msgs

    def Post(self, url, data):
        if self.auto_change_proxy_time > 0:
            self.__class__.auto_change_proxy()
        if not url.startswith("http"):
            url = up.urljoin(self.Domain, url)
        # return self.sess.post(url, data=data, verify=False)
        return self.sess.post(url, data=data, )

    def __getitem__(self, key):
        return self.__class__._data[key]




# import traceback


async def tcp_echo_client(num, host, loop):
    h, p = host.split(":")
    try:
        st = time.time()
        conner = asyncio.open_connection(h, int(p), loop=loop)
        reader, writer = await asyncio.wait_for(conner, timeout=7)
        et = time.time() -st
        # print('Close the socket')
        writer.close()
        return host,et

    except asyncio.TimeoutError:
        return host,9999
    except socket.error as e:
        # traceback.print_stack()
        return host,9999
    # print('Send: %r' % message)
    # writer.write(message.encode())

    # data = yield from reader.read(100)
    # print('Received: %r' % data.decode())


async def _tcp_test(hosts, loop):
    
    task = [tcp_echo_client(i, host, loop) for i, host in enumerate(hosts)]
    return await asyncio.gather(*task)



def TcpTests(hosts):
    loop = asyncio.get_event_loop()
    if loop.is_closed():
        loop = asyncio.new_event_loop()
    res = loop.run_until_complete(_tcp_test(hosts, loop))
    loop.close()
    return [i for i in sorted(res, key=lambda x:x[1]) if i[1] < 100]



 
async def download_coroutine(session, url, filename='', proxy=None):
    with async_timeout.timeout(10):
        async with session.get(url, proxy=proxy) as response:
            if not response.status == 200:
                print("Some error")
                return await response.release()
            
            if not filename:
                filename = os.path.join("/tmp/",os.path.basename(url))
            try:
                with open(filename, 'wb') as f_handle:
                    while True:
                        chunk = await response.content.read(1024)
                        if not chunk:
                            break
                        f_handle.write(chunk)
            except Exception:
                os.remove(filename)
                print(url)
            return await response.release()
 
async def proxy_test(session, url, proxy):
    st = time.time()
    # cprint("[-] " + proxy + " --> " + url, 'yellow')
    try:
        with async_timeout.timeout(13):
            try:
                async with session.get(url, proxy=proxy) as response:
                    status = response.status
                    await response.release()
                    
                    if status == 200:
                        et = time.time() - st
                        cprint("[+]" + proxy + " "+str(et), "green")
            
                        
                        return proxy, et
                    else:
                        
                        return proxy, 9999

            except aiohttp.ClientOSError as e:
                # cprint(e, 'red')
                # cprint("[x] +proxy "+ proxy +" error", 'red')
                return proxy, 9998
            except aiohttp.ServerDisconnectedError as e:
                # cprint(e, 'red')
                # cprint("[x] + ServerDisconnectedError proxy " + proxy + " error", 'red')
                return proxy, 9997
            except Exception as e:
                return proxy, 9996
    except TimeoutError:
        return proxy, 1000
            

async def get_coroutine(session, url, proxy=None):

    with async_timeout.timeout(10):
        async with session.get(url, proxy=proxy) as response:
            if not response.status == 200:
                print("Some error")
                return await response.release()
            
            
            try:
                return await response.text()
            except Exception:
                return await response.release()
 



async def _run(loop, urls, download=False, test=False):
    # auth5 = aiosocks.Socks5Auth('proxyuser1', password='pwd')
    
    # ba = aiohttp.BasicAuth('login')

    # remote resolve
    conn = ProxyConnector()
 
    async with aiohttp.ClientSession(loop=loop, connector=conn, request_class=ProxyClientRequest) as session:
        if download:
            tasks = [download_coroutine(session, url[0], filename, proxy=url[1]) if isinstance(url, list) else  download_coroutine(session, url, filename) for url, filename in urls]
        elif test:
            tasks = [proxy_test(session, url, proxy) for url, proxy in urls]
        else:
            tasks = [get_coroutine(session, url[0], proxy=url[1]) if isinstance(urls, list) else get_coroutine(session, url)  for url in urls]
        return await asyncio.gather(*tasks)
 
def Downloads(urls):
    try:
        loop = asyncio.get_event_loop()
        if loop.is_closed():
            loop = asyncio.new_event_loop()
        res = loop.run_until_complete(_run(loop, urls, download=True))
        loop.close()
        return res
    finally:
        loop.close()

def Gets(urls):
    try:
        loop = asyncio.get_event_loop()
        if loop.is_closed():
            loop = asyncio.new_event_loop()
        res = loop.run_until_complete(_run(loop, urls))
        loop.close()
        return res
    finally:
        loop.close()


def ProxyTests(urls):
    try:
        loop = asyncio.get_event_loop()
        if loop.is_closed():
            loop = asyncio.new_event_loop()
        res = loop.run_until_complete(_run(loop, urls, test=True))
        loop.close()
        return res
    finally:
        loop.close()



class OAuth:
    FORM_CSS_SELECT = []
    LOG_OUT_CSS_SELECT = []
    logined = {}

    def __init__(self,user, passwd, **kargs):
        self.user = user
        self.passwd = passwd
        self.data = {}
        self.method = 'post'

    def _check_form(self, field, type):
        selector = self.__class__.LOG_OUT_CSS_SELECT
        if type == 'login':
            selector = self.__class__.FORM_CSS_SELECT
        found = False
        for form_css_select in selector:
            if field.select(form_css_select):
                inputs = field.select(form_css_select)[0].select("input")
                self.method = field.select(form_css_select)[0].attrs['method']
                # self.data.update({i.attrs['name']:i.attrs['value']
                for i in inputs:
                    if 'name' not in i.attrs:
                        continue
                    if 'username' in i.attrs['name'] or 'email' in i.attrs['name']:
                        self.data[i.attrs['name']] = self.user
                    elif 'passwd' in i.attrs['name'] or 'password' in i.attrs['name']:
                        self.data[i.attrs['name']] = self.passwd
                        
                    if 'value' not in i.attrs:
                        continue

                    self.data[i.attrs['name']] = i.attrs['value']

                self.action = field.select(form_css_select)[0].attrs['action']
                found = True
                break
        return found

    def do_form(self, field, type='login', label=None):
        if self._check_form(field, type):

            if label:
                if not Sess._data[label].sess.cookies:
                    cprint("No cookie found !!!")
                sess = Sess._data[label]
                Post = sess.Post
                Get = sess.Get

            if self.method == 'post':
                return Post(self.action, self.data)
            else:
                return Get(self.action, self.data)
        else:
            print('nichts ist passiert !')

    def login(self, field, label=None, on_done=None, **kargs):
        self.data = kargs

        res = self.do_form(field, label=label)

        if on_done:
            if not hasattr(res, 'select') and hasattr(res, 'text'):
                res = Xml(res.text)
            return on_done(res)
        return res

    def logout(self, field, label=None, on_done=None,**kargs):
        self.data = kargs
        
        res =  self.do_form(field, 'logout',label=label)
        if on_done:
            if not hasattr(res, 'select') and hasattr(res, 'text'):
                res = Xml(res.text)
            return on_done(res)
        return res



# if __name__ == '__main__':
    # loop = asyncio.get_event_loop()
    # loop.run_until_complete(main(loop))


from .proxy import main
from argparse import ArgumentParser


Arger = ArgumentParser()
Arger.add_argument("-u", "--update", default=False, action='store_true', help="update proxy...")
Arger.add_argument("-c", "--country", default='all', help="choose a country")
Arger.add_argument("-l", "--list-code", default=False, action='store_true', help="list country's code")



def run():
	args = Arger.parse_args()
	if args.update:
		main(args.country)
	elif args.list_code:
		main(args.country, help=True)



import socket, sys, os, re, time, datetime
import random
from functools import partial


from .utils import Downloads, Gets
from .utils import load_config
from .utils import Sess
from .utils import Xml
from .utils import OAuth
from .utils import load_proxy
from .utils import up
from .utils import headers
from termcolor import cprint
from qlib.data import dbobj, Cache


tmp_dir = '/tmp/twitter'
if not os.path.exists(tmp_dir):
    os.mkdir(tmp_dir)

text_dir = os.path.join(tmp_dir, 'tw_text')
if not os.path.exists(text_dir):
    os.mkdir(text_dir)

img_dir = os.path.join(tmp_dir, 'tw_img')
if not os.path.exists(img_dir):
    os.mkdir(img_dir)

cache = Cache(os.path.join(tmp_dir, 'users.db'))




sess = Sess('default')
sess_label = 'default'
Sessions = {'default': sess}


def switch_session(label, new=False, **kargs):
    global sess
    global sess_label
    global Sessions

    if label not in Sessions or new is True:
        nsess = Sess(label)
        Sessions[sess_label] = sess
        sess_label = label
        sess = nsess
    else:
        sess_label = label
        sess = Sessions[sess_label]

def Get(url, reset=False, **kargs):
    if not url.startswith("http"):
        url = up.urljoin(Domain, url)
    if reset:
        switch_session(sess_label, new=True)
    return sess.Get(url, **kargs)

def Post(url, data):
    if not url.startswith("http"):
        url = up.urljoin(Domain, url)
    return sess.Post(url, data=data)

## meta area
BASE_URL = "https://twitter.com/"
tw_Url = lambda x: up.urljoin(BASE_URL, x)
open_tw = lambda u: Xml(Get(tw_Url(u)).text)




def time_match(str_time):
    now = time.localtime()
    year = now.tm_year
    mon = now.tm_mon
    day = now.tm_mday
    hour = now.tm_hour
    minu = now.tm_min
    sec = now.tm_sec

    if re.search(r"^(\d{0,4}\-?\d+\-\d+)\s?(\d{0,4}\-?\d+\-\d+)?$", str_time):
            str_time = str_time.strip()
            if ' ' in str_time:
                s,e = str_time.split(' ')
                st = time.strptime(s, '%Y-%m-%d') if s.count('-') == 2 else time.strptime(str(now.tm_year) + '-' + s, '%Y-%m-%d')
                et = time.strptime(e, '%Y-%m-%d') if s.count('-') == 2 else time.strptime(str(now.tm_year) + '-' + e, '%Y-%m-%d')
            else:
                st = time.strptime(str_time, '%Y-%m-%d') if str_time.count('-') == 2 else time.strptime(str(now.tm_year) + '-' + str_time, '%Y-%m-%d')
                et = now
            return time.mktime(st), time.mktime(et)
            
    elif re.search(r'^\d+\s+(day|year|mon|hour)\s+(ago)$' , str_time.strip()):
        num,e ,_  = str_time.split()
        if e == 'year':
            year -= int(num)
        elif e == 'mon':
            mon -= int(num)
        elif e == 'day':
            day -= int(num)
        elif e == 'hour':
            hour -= int(num)

        stamp = datetime.datetime(year, mon, day, hour, minu, sec).timestamp()
        return stamp, time.mktime(now)


class DownCache(dbobj):
    pass

class User(dbobj):
    """
    self.nick = nick
    self.id = id
    self.follow 
    """
    
    def follower(self):
        users = c.query(User, account =self.account)
        return users


class UserCard:
    Data = []
    Data_id = []

    def __init__(self, user_field='', name=None, account=None, desc=None, img_head=None, img_background=None):
        if name and account and desc:
            self.name = name
            self.account = account
            self.desc = desc
            self.img_head = img_head
            self.img_background = img_background

        else:
            try:
                name_a =  user_field.select("a.fullname")[0]

                self.name = name_a.text
                #get account and  extract  'xxxx' from '/xxxx'
                self.account = name_a.attrs['href'][1:]
            except IndexError:
                self.name = user_field.select(".ProfileHeaderCard-nameLink")[0].text
                self.account = user_field.select("b.u-linkComplex-target")[0].text


            try:
                self.desc = user_field.select(".ProfileCard-bio")[0].text
            except IndexError:
                self.desc = user_field.select(".ProfileHeaderCard-bio")[0].text
            
        if self.account not in self.__class__.Data_id:
            self.__class__.Data.append(self)
            self.__class__.Data_id.append(self.account)

    def to_user(self):
        return User(account=self.account, name=self.name, desc=self.desc)

    
    @classmethod
    def load(cls):
        us =  cache.query(User)
        c = 0
        for u in us:
            if u.account not in UserCard.Data_id:
                instance = cls(name=u.name, account=u.account, desc=u.desc)
                c += 1
        print("Load : %d"%c, 'data!')

    @staticmethod
    def save(cache):
        c = 0
        n = []
        ids = [ u.account for u in  cache.query(User)]
        for i in UserCard.Data:
            
            if not i.account in ids:
                n.append(i)
                continue
            c += 1
        cache.save_all(*[i.to_user() for i in n])
        print("Save : %d" % c , "data!")

    # def __del__(self):
    #     self.__class__.save(cache)

    def __getitem__(self, item_id_or_index_or_account_or_time):
        if isinstance(item_id_or_index_or_account_or_time, int):
            return self.__class__.Data[item_id_or_index_or_account_or_time]
        elif isinstance(item_id_or_index_or_account_or_time, str):
            s = []    
            for i in self.__class__.Data:
                if item_id_or_index_or_account_or_time in i.account:
                    s.append(i)
            return s



class Msg(dbobj):
    
    def save_text(self):
        if hasattr(self, 'text'):
            with open(os.path.join(tmp_dir, self.item_id), 'w') as fp:
                fp.write(self.text)


class MessageCard:
    Data = []
    Data_id = []

    def __init__(self, msg_field='', sender='', pub_time=None, item_id=None, name=None, account=None, text='', quote='', media='', retw=0):
        if item_id and name and account:
            self.item_id = item_id
            self.name = name
            self.sender = sender
            self.pub_time = int(pub_time)
            self.account = account
            self.quote = quote
            self.media = media
            self.text = text
            self.retw = retw

        else:
            self.item_id = msg_field.attrs['data-item-id']
            
            self.sender = sender

            # header 
            name_group = msg_field.select('a.account-group')
            if name_group:
                name_group = name_group[0]
            self.pub_time = int(msg_field.select('span.js-short-timestamp')[0].attrs['data-time'])
            self.name = name_group.text
            self.account = name_group.attrs['href'][1:]
            

            # text area
            self.text_group = msg_field.select('.js-tweet-text-container')[0]
            self.text = self.text_group.text

            
            # quote
            self.quote = ''
            quote = msg_field.select('a.QuoteTweet-link')
            if quote:
                self.quote = quote[0].attrs['data-conversation-id']

            # media
            self.media = ''
            m = msg_field.select('.js-macaw-cards-iframe-container')
            if m:
                self.media = m[0].attrs['data-card-url']

            self.retw = 1 if sender != self.account else 0

        if self.item_id not in self.__class__.Data_id:
            self.__class__.Data.append(self)
            self.__class__.Data_id.append(self.item_id)




    def to_db(self):
        return Msg(item_id=self.item_id, sender=self.sender, name=self.name, account=self.account, retw=self.retw, pub_time=int(self.pub_time), quote=self.quote, media=self.media)


    def __repr__(self):
        return '|'.join([self.sender, "tw : ", self.item_id, self.name ,self.account])

    @staticmethod
    def save(cache):
        c = 0
        ids = [ u.item_id for u in  cache.query(Msg)]
        n = []
        for i in MessageCard.Data:
            
            if i.item_id not in ids:
                n.append(i)
                c += 1
                continue

            
        _cache = [i.to_db() for i in n]
        cache.save_all(*_cache)
        print("Save : %d" % c , "data!")
        [i.save_text() for i in _cache]


    @classmethod
    def load(cls):
        us =  cache.query(Msg)
        c = 0
        for u in us:
            if u.item_id not in MessageCard.Data_id:
                instance = cls(item_id= u.item_id, name=u.name, pub_time=u.pub_time, account=u.account, retw=u.retw, sender=u.sender, quote=u.quote, media=u.media)
                if os.path.exists(os.path.join(text_dir, u.item_id)):
                    text = ''
                    with open(os.path.join(text_dir, u.item_id)) as fp:
                        text = fp.read()
                    instance.text = text

                c += 1
        print("Load : %d"%c, 'data!')

    # def __del__(self):
    #     self.__class__.save(cache)

    def __getitem__(self, item_id_or_index_or_account_or_time):
        if isinstance(item_id_or_index_or_account_or_time, int):
            return self.__class__.Data[item_id_or_index_or_account_or_time]
        elif isinstance(item_id_or_index_or_account_or_time, str):
            s = []
            if re.match(r'^\d{10,18}$',item_id_or_index_or_account_or_time):
                
                for num, i in enumerate(self.__class__.Data_id):
                    if item_id_or_index_or_account_or_time in i:
                        s.append(self.__class__.Data[num])
                return s

            elif time_match(item_id_or_index_or_account_or_time):
                stamps = time_match(item_id_or_index_or_account_or_time)
                st = min(stamps)
                et = max(stamps)

                for i in self.__class__.Data:
                    if i.pub_time > st and i.pub_time < et:
                        s.append(i)
                return s
            else:
                for i in self.__class__.Data:
                    if item_id_or_index_or_account_or_time in i.account:
                        s.append(i)
                return s





class TwitterAuth(OAuth):
    BASE_URL = "https://twitter.com/"
    
    FORM_CSS_SELECT = [
        'form.LoginForm',
        'form.js-signin',
    ]

    LOG_OUT_CSS_SELECT = [
        'signout-form',
    ]

    

    def login(self, tw_panel):
        return super().login(tw_panel.tw, tw_panel.sess_label, on_done=tw_panel.check_access)

    def logout(self, tw_panel):
        return super().login(tw_panel.tw, tw_panel.sess_label)        


class TwitterPanel:

    def __init__(self, user, account=None, name=None, msg_pos=None, follow_pos=None, new_session=False, proxy=True):
        self.sess_label = 'twitter|' + user
        if new_session:
            self.sess_label = 'twitter' + str(time.time())
        self.follow_pos = 0
        self.max_position = 0
        if proxy == 'pub':
            proxy = random.choice(load_proxy()[:10])
        self.sess = Sess(self.sess_label, proxy=proxy)
        
        if account:
            self.account = account
            self.name = name
            self.max_position = msg_pos
            self.follow_pos = follow_pos
            self.tw = None
        else:
            self.tw = Xml(self.sess.Get(tw_Url(user)).text)
            self.account = user
            self.name = self.tw.select('a.ProfileHeaderCard-nameLink')[0].text.strip()
            self.max_position = self.tw.select(".stream-container")[0].attrs['data-max-position']
            self.desc = self.tw.select('.ProfileHeaderCard-bio')[0].text

            # load user..
            u = UserCard(self.tw.select(".ProfileHeaderCard")[0])
            # load user's img
            img_head = self.tw.select("img.ProfileAvatar-image")[0].attrs['src']
            u.img_head = img_head



    def follower(self, deep=3):
        if self.follow_pos == 0:
            b_follow_ur = '/%s/following' % self.account
            users = []
            init = Xml(self.sess.Get(b_follow_ur).text)
            if not self.check_access(init):
                return
            ## init followers
            position = init.select(".GridTimeline-items")[0].attrs['data-min-position']
            if int(position) < int(self.follow_pos):
                position = str(self.follow_pos)

            for w in init.select(".ProfileCard"):
                u = UserCard(w)
                print(u.name)

        headers = self.sess.sess.headers
        headers.update({
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Referer": "https://twitter.com/%s/following" % self.account
        })
        #next
        position = self.follow_pos
        for i in range(deep):
            # print(i, 'turn', end='\r')
            follow_ur = '/%s/following/users?include_available_features=1&include_entities=1&max_position=%s&reset_error_state=false' % (self.account, position)
            res = self.sess.Get(tw_Url(follow_ur), headers=headers).json()
            
            position = res['min_position']
            if not res['has_more_items']:
                print(position)
                break
            new_users = Xml(res['items_html']).select('.ProfileCard')
            
            for w in new_users:
                u = UserCard(w)
                print(i,'/',u.name.strip(), end='\r')
            self.follow_pos = position

    def message(self, deep=5):
        msg_count = 0
        if self.tw:
            # init msg card , if load cache will skip this.
            if not self.check_access(self.tw):
                return
            messgs = self.tw.select('li.stream-item')
            for msg in messgs:
                name_group = msg.select('a.account-group')
                if not name_group:
                    continue
                pub_item = msg.select('span.js-short-timestamp')
                if not pub_item:
                    continue

                MessageCard(msg, self.account)
                msg_count += 1

        position = self.max_position
        # more 
        headers.update({
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Referer": "https://twitter.com/%s" % self.account
        })

        for i in range(deep):
            more_msg = '/i/profiles/show/ggreenwald/timeline/tweets?include_available_features=1&include_entities=1&max_position=%s&reset_error_state=false' %  self.max_position
            u = tw_Url(more_msg)
            res = self.sess.Get(tw_Url(u), headers=headers).json()
            # yield res
            
            position = res['min_position']
            if not res['has_more_items']:
                print(res['min_position'])
                break
            for msg in Xml(res['items_html']).select('li.stream-item'):
                name_group = msg.select('a.account-group')
                if not name_group:
                    continue
                pub_item = msg.select('span.js-short-timestamp')
                if not pub_item:
                    continue

                MessageCard(msg, self.account)
                msg_count += 1
            print( msg_count,'/',i+1, 'turn', end='\r')
            self.max_position = position

    def check_access(self ,login_res):

        forms = login_res.select("form")
        for f in forms:
            if 'action' not in f.attrs:
                continue

            if f.attrs['action'] == '/account/access':
                cprint('may account: %s is Limited!!' % self.auth.user)
                return False
        return True

    @classmethod
    def load(cls, user='', load_all=False):
        '''
        if user == '':
            will load all user!!

        if load_all:
            will load all data include MessagCard, UserCard.
        '''
        if not user:
            users = cache.query(User)
            ts = []
            for user in users:
                ts.append(TwitterPanel.load(user.account, load_all=load_all))
            return ts

        o = cache.query_one(User, account=user)
        if not o:
            print("Not found user:%s in local DB." % user)
            return 

        d = cache.query_one(DownCache, account=user)
        if load_all:
            print("MessageCard load.")
            MessageCard.load()
            print("UserCard load.")
            UserCard.load()

        return TwitterPanel(o.account, 
            account=o.account,
            name=o.name,
            msg_pos=d.msg_pos,
            follow_pos=d.follow_pos)

    def __getitem__(self, key):
        """
        key:  
            'u:xxx' = search UserCard
            'm:xxx' = search MessageCard
                xxx : can be replace with a fuzy account, a name
                    if in 'm:' mode:
                        xxx can be use a time str like:
                            '222 day ago ' , '1 year ago' ,'12-2' , '2016-1-1 3-2' (2016-1-1 ~ now)

        """
        if ':' not in key:
            raise KeyError("""
            key:  
            'u:xxx' = search UserCard
            'm:xxx' = search MessageCard
                xxx : can be replace with a fuzy account, a name
                    if in 'm:' mode:
                        xxx can be use a time str like:
                            '222 day ago ' , '1 year ago' ,'12-2' , '2016-1-1 3-2' (2016-1-1 ~ now)
            """)

        mode, key = key.split(":")
        if re.search(r'^\d*$', key):
            key = int(key)

        if mode == 'm':
            return MessageCard.Data[0][key]

        elif mode == 'u':
            return UserCard.Data[0][key]


    def load_cache(self):
        o = cache.query_one(DownCache, account=self.account)
        if o:
            self.max_position = o.msg_pos
            self.follow_pos = o.follow_pos


        MessageCard.load()
        UserCard.load()

    def save_cache(self):
        o = cache.query_one(DownCache, account=self.account)
        
        # save cache data
        MessageCard.save(cache)
        UserCard.save(cache)

        if not o:
            w = DownCache(account= self.account, msg_pos=self.max_position, follow_pos=self.follow_pos)
            w.save(cache)
            return

        fo_pos = o.follow_pos
        msg_pos = o.msg_pos
        changed = False
        if msg_pos < int(self.max_position):
            msg_pos = self.max_position
            changed = True

        if fo_pos < int(self.follow_pos):
            fo_pos = self.follow_pos
            changed = True

        if changed:
            w = DownCache(account= self.account, msg_pos=str(msg_pos), follow_pos=str(fo_pos))
            cache.delete(o)
            cache.save(w)

    def login(self, user,passwd):
        self.auth = TwitterAuth(user, passwd)
        return self.auth.login(self)

    def logout(self):
        if hasattr(self, 'auth'):
            return self.auth.logout(self)

    @classmethod
    def Login(cls, user,passwd):
        t = cls('NBA')
        t.login(user, passwd)
        return t


    def __del__(self):
        
        self.logout()

class Mail:

    def __init__(self):
        self.sess = Sess("mail")
        self.menu = Xml(self.sess.Get('https://10minutemail.net/').text)
        self.start_time = time.time()
        self.address = 'X'
        address_area = self.menu.select(".mailtext")
        if address_area:
            self.address = address_area[0].attrs['value']

        self.recve = []

    def table(self):
        rows = iter(Xml(self.sess.Get("https://10minutemail.net/mailbox.ajax.php").text).select("table")[0])
        # rows = iter(self.menu.select("table"))
        headers = [col.text for col in next(rows)] + ['link']
        msgs = []
        for row in rows:
            values = [col.text for col in row]
            values += [row.attrs['onclick'].split("'")[1]]
            msgs.append(dict(zip(headers, values)))
        self.recve = msgs
        return msgs

    def read(self, index):
        if isinstance(index, str):
            for m in self.recve:
                w = Xml(self.sess.Get(m['link']).text).select(".tab_content")[1]
                links = w.select("a")
                for a in links:
                    if index in a.text:
                        return self.sess.Get(a.attrs.get("href"))
            return
        m = self.recve[index]
        w = Xml(self.sess.Get(m['link']).text).select(".tab_content")[1]
        cprint(m, 'red')
        print(w.text)
        imgs = w.select("img")
        print("--------")
        print("Imgs:", imgs)
        links = w.select("a")
        for a in links:
            cprint(a.text + " ---> " + a.attrs.get("href",""), 'yellow')
            r = ''
            if not a.attrs.get('href'):
                continue
            while not r in ['Y','n']:
                r = input("[redirect? [Y/n]]")
                if not r:
                    r = 'n'

            if r == 'Y':
                return self.sess.Get(a.attrs.get("href"))


    def refresh(self):
        self.menu = Xml(self.sess.Get('https://10minutemail.net/').text)

    def more_10_minute(self):
        self.sess.Get('https://10minutemail.net/more.html')
        self.refresh()


    def time(self):
        return str(time.time() - self.start_time)

    def __repr__(self):
        return self.address + "|" + self.time()
        


import os

from setuptools import setup, find_packages
from setuptools.command.install import install

import os, json
E = os.path.exists
J = os.path.join
MK = os.mkdir

ROOT_C = "/usr/local/etc/SocialOrigan"
INI = """
[path]
ROOT=/usr/local/etc/SocialOrigan

[proxy config]
type=socks5h
hostport=127.0.0.1:1080

[proxy 1]
type=socks5h
hostport=223.197.203.41:15356
"""

def initial():

	if not E(ROOT_C):
	    print("// init")
	    MK(ROOT_C)

	    with open(J(ROOT_C, "config.ini"), "w") as fp:
	        fp.write(INI)


	    codes = {
	        "ALL": "All:0",
	        "AD": "Andorra:1",
	        "AE": "United Arab Emirates:2",
	        "AF": "Afghanistan:3",
	        "AG": "Antigua and Barbuda:4",
	        "AI": "Anguilla:5",
	        "AL": "Albania:6",
	        "AM": "Armenia:7",
	        "AN": "Netherlands Antilles:8",
	        "AO": "Angola:9",
	        "AQ": "Antarctica:10",
	        "AR": "Argentina:11",
	        "AS": "American Samoa:12",
	        "AT": "Austria:13",
	        "AU": "Australia:14",
	        "AW": "Aruba:15",
	        "AX": "Aland Islands:16",
	        "AZ": "Azerbaijan:17",
	        "BA": "Bosnia and Herzegovina:18",
	        "BB": "Barbados:19",
	        "BD": "Bangladesh:20",
	        "BE": "Belgium:21",
	        "BF": "Burkina Faso:22",
	        "BG": "Bulgaria:23",
	        "BH": "Bahrain:24",
	        "BI": "Burundi:25",
	        "BJ": "Benin:26",
	        "BL": "Saint Barthelemy:27",
	        "BM": "Bermuda:28",
	        "BN": "Brunei Darussalam:29",
	        "BO": "Bolivia:30",
	        "BR": "Brazil:31",
	        "BS": "Bahamas:32",
	        "BT": "Bhutan:33",
	        "BV": "Bouvet Island:34",
	        "BW": "Botswana:35",
	        "BY": "Belarus:36",
	        "BZ": "Belize:37",
	        "CA": "Canada:38",
	        "CC": "Cocos (Keeling) Islands:39",
	        "CD": "Congo:40",
	        "CF": "Central African Republic:41",
	        "CG": "Congo:42",
	        "CH": "Switzerland:43",
	        "CI": "Cote d`Ivoire:44",
	        "CK": "Cook Islands:45",
	        "CL": "Chile:46",
	        "CM": "Cameroon:47",
	        "CN": "China:48",
	        "CO": "Colombia:49",
	        "CR": "Costa Rica:50",
	        "CU": "Cuba:51",
	        "CV": "Cape Verde:52",
	        "CX": "Christmas Island:53",
	        "CY": "Cyprus:54",
	        "CZ": "Czech Republic:55",
	        "DE": "Germany:56",
	        "DJ": "Djibouti:57",
	        "DK": "Denmark:58",
	        "DM": "Dominica:59",
	        "DO": "Dominican Republic:60",
	        "DZ": "Algeria:61",
	        "EC": "Ecuador:62",
	        "EE": "Estonia:63",
	        "EG": "Egypt:64",
	        "EH": "Western Sahara:65",
	        "ER": "Eritrea:66",
	        "ES": "Spain:67",
	        "ET": "Ethiopia:68",
	        "EU": "European Union:69",
	        "FI": "Finland:70",
	        "FJ": "Fiji:71",
	        "FK": "Falkland Islands:72",
	        "FM": "Micronesia:73",
	        "FO": "Faroe Islands:74",
	        "FR": "France:75",
	        "GA": "Gabon:76",
	        "GB": "United Kingdom:77",
	        "GD": "Grenada:78",
	        "GE": "Georgia:79",
	        "GF": "French Guiana:80",
	        "GG": "Guernsey:81",
	        "GH": "Ghana:82",
	        "GI": "Gibraltar:83",
	        "GL": "Greenland:84",
	        "GM": "Gambia:85",
	        "GN": "Guinea:86",
	        "GP": "Guadeloupe:87",
	        "GQ": "Equatorial Guinea:88",
	        "GR": "Greece:89",
	        "GS": "South Georgia and the South Sandwich Islands:90",
	        "GT": "Guatemala:91",
	        "GU": "Guam:92",
	        "GW": "Guinea-Bissau:93",
	        "GY": "Guyana:94",
	        "HK": "Hong Kong:95",
	        "HM": "Heard Island and McDonald Islands:96",
	        "HN": "Honduras:97",
	        "HR": "Croatia:98",
	        "HT": "Haiti:99",
	        "HU": "Hungary:100",
	        "ID": "Indonesia:101",
	        "IE": "Ireland:102",
	        "IL": "Israel:103",
	        "IM": "Isle of Man:104",
	        "IN": "India:105",
	        "IO": "British Indian Ocean Territory:106",
	        "IQ": "Iraq:107",
	        "IR": "Iran:108",
	        "IS": "Iceland:109",
	        "IT": "Italy:110",
	        "JE": "Jersey:111",
	        "JM": "Jamaica:112",
	        "JO": "Jordan:113",
	        "JP": "Japan:114",
	        "KE": "Kenya:115",
	        "KG": "Kyrgyzstan:116",
	        "KH": "Cambodia:117",
	        "KI": "Kiribati:118",
	        "KM": "Comoros:119",
	        "KN": "Saint Kitts and Nevis:120",
	        "KP": "North Korea:121",
	        "KR": "South Korea:122",
	        "KW": "Kuwait:123",
	        "KY": "Cayman Islands:124",
	        "KZ": "Kazakhstan:125",
	        "LA": "Lao People`s Democratic Republic:126",
	        "LB": "Lebanon:127",
	        "LC": "Saint Lucia:128",
	        "LI": "Liechtenstein:129",
	        "LK": "Sri Lanka:130",
	        "LR": "Liberia:131",
	        "LS": "Lesotho:132",
	        "LT": "Lithuania:133",
	        "LU": "Luxembourg:134",
	        "LV": "Latvia:135",
	        "LY": "Libyan Arab Jamahiriya:136",
	        "MA": "Morocco:137",
	        "MC": "Monaco:138",
	        "MD": "Moldova:139",
	        "ME": "Montenegro:140",
	        "MF": "Saint Martin:141",
	        "MG": "Madagascar:142",
	        "MH": "Marshall Islands:143",
	        "MK": "Macedonia:144",
	        "ML": "Mali:145",
	        "MM": "Myanmar:146",
	        "MN": "Mongolia:147",
	        "MO": "Macao:148",
	        "MP": "Northern Mariana Islands:149",
	        "MQ": "Martinique:150",
	        "MR": "Mauritania:151",
	        "MS": "Montserrat:152",
	        "MT": "Malta:153",
	        "MU": "Mauritius:154",
	        "MV": "Maldives:155",
	        "MW": "Malawi:156",
	        "MX": "Mexico:157",
	        "MY": "Malaysia:158",
	        "MZ": "Mozambique:159",
	        "NA": "Namibia:160",
	        "NC": "New Caledonia:161",
	        "NE": "Niger:162",
	        "NF": "Norfolk Island:163",
	        "NG": "Nigeria:164",
	        "NI": "Nicaragua:165",
	        "NL": "Netherlands:166",
	        "NO": "Norway:167",
	        "NP": "Nepal:168",
	        "NR": "Nauru:169",
	        "NU": "Niue:170",
	        "NZ": "New Zealand:171",
	        "OM": "Oman:172",
	        "PA": "Panama:173",
	        "PE": "Peru:174",
	        "PF": "French Polynesia:175",
	        "PG": "Papua New Guinea:176",
	        "PH": "Philippines:177",
	        "PK": "Pakistan:178",
	        "PL": "Poland:179",
	        "PM": "Saint Pierre and Miquelon:180",
	        "PN": "Pitcairn:181",
	        "PR": "Puerto Rico:182",
	        "PS": "Palestinian Territory:183",
	        "PT": "Portugal:184",
	        "PW": "Palau:185",
	        "PY": "Paraguay:186",
	        "QA": "Qatar:187",
	        "RE": "Reunion:188",
	        "RO": "Romania:189",
	        "RS": "Serbia:190",
	        "RU": "Russian Federation:191",
	        "RW": "Rwanda:192",
	        "SA": "Saudi Arabia:193",
	        "SB": "Solomon Islands:194",
	        "SC": "Seychelles:195",
	        "SD": "Sudan:196",
	        "SE": "Sweden:197",
	        "SG": "Singapore:198",
	        "SH": "Saint Helena:199",
	        "SI": "Slovenia:200",
	        "SJ": "Svalbard and Jan Mayen:201",
	        "SK": "Slovakia:202",
	        "SL": "Sierra Leone:203",
	        "SM": "San Marino:204",
	        "SN": "Senegal:205",
	        "SO": "Somalia:206",
	        "SR": "Suriname:207",
	        "ST": "Sao Tome and Principe:208",
	        "SU": "Soviet Union (being phased out):209",
	        "SV": "El Salvador:210",
	        "SY": "Syrian Arab Republic:211",
	        "SZ": "Swaziland:212",
	        "TC": "Turks and Caicos Islands:213",
	        "TD": "Chad:214",
	        "TF": "French Southern Territories:215",
	        "TG": "Togo:216",
	        "TH": "Thailand:217",
	        "TJ": "Tajikistan:218",
	        "TK": "Tokelau:219",
	        "TL": "Timor-Leste:220",
	        "TM": "Turkmenistan:221",
	        "TN": "Tunisia:222",
	        "TO": "Tonga:223",
	        "TP": "Portuguese Timor (being phased out):224",
	        "TR": "Turkey:225",
	        "TT": "Trinidad and Tobago:226",
	        "TV": "Tuvalu:227",
	        "TW": "Taiwan:228",
	        "TZ": "Tanzania:229",
	        "UA": "Ukraine:230",
	        "UG": "Uganda:231",
	        "UK": "United Kingdom:232",
	        "UM": "United States Minor Outlying Islands:233",
	        "US": "United States:234",
	        "UY": "Uruguay:235",
	        "UZ": "Uzbekistan:236",
	        "VA": "Holy See (Vatican City State):237",
	        "VC": "Saint Vincent and the Grenadines:238",
	        "VE": "Venezuela:239",
	        "VG": "Virgin Islands, British:240",
	        "VI": "Virgin Islands, U.S.:241",
	        "VN": "VietNam:242",
	        "VU": "Vanuatu:243",
	        "WF": "Wallis and Futuna:244",
	        "WS": "Samoa:245",
	        "YE": "Yemen:246",
	        "YT": "Mayotte:247",
	        "YU": "Yugoslavia:248",
	        "ZA": "South Africa:249",
	        "ZM": "Zambia:250",
	        "ZW": "Zimbabwe:251",
	        "A1": "Anonymous Proxies:252",
	        "A2": "Satellite Providers:253",
	        "AP": "Asia/Pacific Region:254",
	        "O1": "Other Country:255",
	        "CW": "Curacao:256",
	        "CS": "Czechoslovakia:257",
	        "SX": "Sint Maarten:258",
	        "SS": "South Sudan:259"
	    }

	    json.dump(codes, open(J(ROOT_C, 'country_code.json'), 'w'))


class MyInstall(install):

    def run(self):
        initial()
        print(" ... ...............")
        install.run(self)
        

# in the setup function:
# cmdclass={}

setup(name='SocialOrgan',
    version='0.1',
    cmdclass={
        'install': MyInstall,
    },
    description='a simple way to ...',
    url='https://github.com/xxx/xx.git',
    author='xxx',
    author_email='darkhackdevil@gmail.com',
    license='MIT',
    zip_safe=False,
    packages=find_packages(),
    install_requires=['aiohttp==2.3.10',
    	'aiosocks==0.2.6',
    	'bs4',
    	'lxml',
    	'requests',
    	'pysocks5',
    	'termcolor',
    	'PySocks==1.6.7',
    	'pyexecjs==1.5.1',
    	'mroylib-min==1.4.1',
    ],
    entry_points={
        'console_scripts': ['freeproxy=SocialOrigan.cmd:run']
    },



)





import sys,os
import re
import json
import time
import configparser

from .utils import Gets, Sess, Xml, TcpTests, ProxyTests
from .utils import load_config
from .utils import load_proxy
import execjs
from termcolor import cprint

PROXY_HOSTS = "http://premiumproxy.net/"
Config = load_config()
sess = Sess("proxy")
ROOT = Config.get("path", "ROOT")
class Data:
    pass

class Premiumproxy:
    codes = json.load(open(os.path.join(ROOT, "country_code.json")))

    def __init__(self, num='hk', types=''):
        coun, num = self.__class__.codes[num.upper()].split(":")
        self.data = {
            'tldc':num,
            'anmm':2,
            'vlast':0,
            'eport':'',
            'submit':'Search'
        }


    def pull(self,country):
        coun, self.data['tldc'] = self.__class__.codes[country.upper()].split(":")
        print(coun)
        if type == "socks":
            self.base = Xml(sess.Get("http://premiumproxy.net/socks-proxy-list.php").text)
        else:    
            self.base = Xml(sess.Post("http://premiumproxy.net/search-proxy.php", data=self.data).text)

    def load(self, country='all'):
        cprint("[-] Pull data from server ...", 'red')
        self.pull(country)
        
        cprint("[-] Load js Context...", 'red')
        self.get_js_context()

        cprint("[+] update proxy list ...", 'green')
        self.proxy = self.get_proxy_list()
        if self.proxy:
            cprint("[-] test proxy server speed ....", 'red')
            self.test_proxy_connect()
            time.sleep(1)
            cprint("[-] test proxy connect to Google's speed ....", 'red')
            self.test_proxy_google()

    def parse(self, i):

        i = i[1:-1]
        f,l = i.split("^")
        return self.e.__dict__[f] ^ self.e.__dict__[l]

    def get_js_context(self):
        t = 0
        while  1:
            
            try:
                s = self.base.select("script")[5].text.split("eval(")[1].strip()[:-1]
                self.js_context =  execjs.eval(s)
                e = Data()
                exec(self.js_context)
                for i in self.js_context.split(";"):
                    if not i.strip(): continue

                    k , v = i.split("=")
                    try:   
                        setattr(e, k, eval(v))
                    except NameError:
                        f, l = v.split("^")

                        if re.match('\d', f):
                            f = "e." + f

                        if re.match('\d', l):
                            l = "e." + l



                        setattr(e,k, eval(f+"^" + l))

                    
                self.e = e
                break
            except IndexError as e:
                t += 1
                cprint("the page is error, ... try %d time" % t, "red")
                cprint("[-] Pull again ...", 'red')
                self.pull(country='All')
                if t >= 5:
                    cprint("reload failed !! ", "red")
                    break
                continue

    def test_proxy_google(self):
        proxyies = load_proxy()
        proxies = [['https://www.google.com',i] for i in  load_proxy() if not i.startswith("https")]
        tmp =  ProxyTests(proxies)
        test_res = [i for i in tmp if i[1] < 20 ]
        sort_proxy = sorted(test_res, key=lambda x:x[1])
        config = configparser.ConfigParser()
        for i,e  in enumerate(sort_proxy):
            t,p = e[0].split("://")
            config[str(i)] = {
                'hostport':p,
                'type':t.upper(),
                'speed': e[1]
            }

        cprint("[+] sorted proxy to Proxy.ini", 'green', attrs=['bold'])
        with open(os.path.join(ROOT, 'Proxy.ini'), 'w') as fp:
            config.write(fp)

    def test_proxy_connect(self):
        d = {i['Proxy address:port']:i['Proxy type'] for i in self.proxy}
        res = TcpTests(d.keys())
        config = configparser.ConfigParser()
        for i,e  in enumerate(res):
            config[str(i)] = {
                'hostport':e[0],
                'type':d[e[0]],
                'speed': e[1]
            }

        cprint("[+] cache proxy to Proxy.ini", 'green', attrs=['bold'])
        with open(os.path.join(ROOT, 'Proxy.ini'), 'w') as fp:
            config.write(fp)


    def choose_a_proxy(self):
        p_type = Config.get('proxy '+ str(num), 'type')
        p_host_port = Config.get('proxy '+ str(num), 'hostport')
        return p_type, p_host_port


    def get_proxy_list(self):
        tables = sess.Table(self.base)

        for i in tables:
            if len(i) > 30:
                k = 'Proxy address:port'

                table = i
                st = time.time()
                for row in table:
                    host_port = row[k]
                    
                    if 'document' not in host_port:
                        continue
                    b = host_port.split()[1]
                    a = b.split("document")[0]
                    row[k] = a + ":" + ''.join([ str(self.parse(ii)) for ii in re.findall(r'(\([\w\^]*\))', host_port)])
                    # print(row[k])
                et = time.time() -st
                print("Use time: ", et)
                return table
        cprint("no proxy found ? ", 'red')
        return False

def main(country='all', help=False):
    if help:
        for i in Premiumproxy.codes:
            cprint(i, 'green')
        return
    if country.upper() not in Premiumproxy.codes:
        return
        
    p = Premiumproxy()
    p.load(country)

if __name__ == '__main__':
    if len(sys.argv) == 2:
        main(sys.argv[1])
    elif len(sys.argv) == 1:
        main()
            
